@@ -29,7 +29,10 @@ subject to the following restrictions:
 #include "LinearMath/btVector3.h"
 #include "LinearMath/btAlignedAllocator.h"
 
-
+#ifdef BT_DEBUG
+#  include <iostream>
+#endif
+#include <cstdio>
 //http://msdn.microsoft.com/library/default.asp?url=/library/en-us/vclang/html/vclrf__m128.asp
 
 
@@ -351,7 +354,23 @@ public:
 	{
 
 		btAssert(m_useQuantization);
-
+#ifdef BT_DEBUG
+		if( (point.getX() > m_bvhAabbMax.getX() ) ||
+		    (point.getY() > m_bvhAabbMax.getY() ) ||
+		    (point.getZ() > m_bvhAabbMax.getZ() ) ||
+		    (point.getX() < m_bvhAabbMin.getX() ) ||
+		    (point.getY() < m_bvhAabbMin.getY() ) ||
+		    (point.getZ() < m_bvhAabbMin.getZ() )    )
+        {
+            std::cout << "Bullet Assertion error " 
+                << point.getX() << " "<<point.getY()<<" "<<point.getZ()
+                << "  "<<m_bvhAabbMax.getX()<<" "<< m_bvhAabbMax.getY()
+                << " " <<m_bvhAabbMax.getZ()<<"  "
+                << m_bvhAabbMin.getX()<<" "<< m_bvhAabbMin.getX()
+                << " "<< m_bvhAabbMin.getX()<<" "
+                << isMax;
+        }
+#endif
 		btAssert(point.getX() <= m_bvhAabbMax.getX());
 		btAssert(point.getY() <= m_bvhAabbMax.getY());
 		btAssert(point.getZ() <= m_bvhAabbMax.getZ());
@@ -424,7 +443,6 @@ public:
 		clampedPoint.setMin(m_bvhAabbMax);
 
 		quantize(out,clampedPoint,isMax);
-
 	}
 	
 	SIMD_FORCE_INLINE btVector3	unQuantize(const unsigned short* vecIn) const
