@@ -21,20 +21,24 @@
 #include <stdexcept>
 #include <sstream>
 
-#include "grand_prix_data.hpp"
-#include "grand_prix_manager.hpp"
 #include "karts/kart.hpp"
-#include "lisp/lisp.hpp"
-#include "lisp/parser.hpp"
+#include "karts/kart_properties_manager.hpp"
 #include "modes/linear_world.hpp"
+#include "race/grand_prix_data.hpp"
+#include "race/grand_prix_manager.hpp"
+#include "race/race_manager.hpp"
 #include "tracks/track.hpp"
 #include "tracks/track_manager.hpp"
+#include "utils/translation.hpp"
 
 ChallengeData::ChallengeData(const std::string& filename)
+#ifndef WIN32
+                                                      throw(std::runtime_error)
+#endif
 {
     m_filename    = filename;
     m_major       = RaceManager::MAJOR_MODE_SINGLE;
-    m_minor       = RaceManager::MINOR_MODE_QUICK_RACE;
+    m_minor       = RaceManager::MINOR_MODE_NORMAL_RACE;
     m_difficulty  = RaceManager::RD_EASY;
     m_num_laps    = -1;
     m_num_karts   = -1;
@@ -44,20 +48,18 @@ ChallengeData::ChallengeData(const std::string& filename)
     m_gp_id       = "";
     m_energy      = -1;
 
-    lisp::Parser parser;
-    const lisp::Lisp* const ROOT = parser.parse(filename);
-
-    const lisp::Lisp* const lisp = ROOT->getLisp("challenge");
-    if(!lisp)
+    XMLNode *root = new XMLNode( filename );
+//    if(!root || root->getName()!="challenges")
+    if(!root || root->getName()!="challenge")
     {
-        delete ROOT;
+        delete root;
         std::ostringstream msg;
         msg << "Couldn't load challenge '" << filename << "': no challenge node.";
         throw std::runtime_error(msg.str());
     }
 
     std::string mode;
-    lisp->get("major", mode);
+    root->get("major", &mode);
 
     if(mode=="grandprix")
         m_major = RaceManager::MAJOR_MODE_GRAND_PRIX;
@@ -66,30 +68,37 @@ ChallengeData::ChallengeData(const std::string& filename)
     else
         error("major");
 
-    lisp->get("minor", mode);
+    root->get("minor", &mode);
     if(mode=="timetrial")
         m_minor = RaceManager::MINOR_MODE_TIME_TRIAL;
     else if(mode=="quickrace")
-        m_minor = RaceManager::MINOR_MODE_QUICK_RACE;
+        m_minor = RaceManager::MINOR_MODE_NORMAL_RACE;
     else if(mode=="followtheleader")
         m_minor = RaceManager::MINOR_MODE_FOLLOW_LEADER;
     else
         error("minor");
+
     std::string s;
-    if(!lisp->get("name", s)             ) error("name");
-    setName(s);
-    if(!lisp->get("id", s)               ) error("id");
+    if(!root->get("name", &s) ) error("name");
+    //std::cout << "    // Challenge name = <" << s.c_str() << ">\n";
+    setName( _(s.c_str()) );
+
+    if(!root->get("id", &s) ) error("id");
     setId(s);
-    if(!lisp->get("description", s)      ) error("description");
-    setChallengeDescription(s); 
-    if(!lisp->get("karts", m_num_karts)  ) error("karts");
+
+    if(!root->get("description", &s) ) error("description");
+    setChallengeDescription( _(s.c_str()) );
+    //std::cout << "    // Challenge description = <" << s.c_str() << ">\n";
+
+    if(!root->get("karts", &m_num_karts)  ) error("karts");
+
     // Position is optional except in GP and FTL
-    if(!lisp->get("position", m_position) &&
+    if(!root->get("position", &m_position) &&
        //RaceManager::getWorld()->areKartsOrdered() ) // FIXME - order and optional are not the same thing
         (m_minor==RaceManager::MINOR_MODE_FOLLOW_LEADER ||
          m_major==RaceManager::MAJOR_MODE_GRAND_PRIX))
                                            error("position");
-    lisp->get("difficulty", s);
+    root->get("difficulty", &s);
     if(s=="easy")
         m_difficulty = RaceManager::RD_EASY;
     else if(s=="medium")
@@ -99,32 +108,52 @@ ChallengeData::ChallengeData(const std::string& filename)
     else
         error("difficulty");
 
-    lisp->get("time",       m_time       );  // one of time/position
-    lisp->get("position",   m_position   );  // must be set
+    root->get("time", &m_time );  // one of time/position
+
+    root->get("position", &m_position );  // must be set
     if(m_time<0 && m_position<0) error("position/time");
-    lisp->get("energy", m_energy     ); // This is optional
+
+    root->get("energy", &m_energy ); // This is optional
     if(m_major==RaceManager::MAJOR_MODE_SINGLE)
     {
-        if(!lisp->get("track",  m_track_name )) error("track");
-        if(!lisp->get("laps",   m_num_laps   ) && 
-           m_minor!=RaceManager::MINOR_MODE_FOLLOW_LEADER)
+        if (!root->get("track",  &m_track_name ))
+        {
+            error("track");
+        }
+        if (track_manager->getTrack(m_track_name) == NULL)
+        {
+            error("track");
+        }
+        
+        if (!root->get("laps",   &m_num_laps   ) && m_minor!=RaceManager::MINOR_MODE_FOLLOW_LEADER)
+        {
            error("laps");
+        }
     }
     else   // GP
     {
-        if(!lisp->get("gp",   m_gp_id )) error("gp");
+        if (!root->get("gp",   &m_gp_id ))                     error("gp");
+        if (grand_prix_manager->getGrandPrix(m_gp_id) == NULL) error("gp");
     }
 
-    getUnlocks(lisp, "unlock-track",      UNLOCK_TRACK);
-    getUnlocks(lisp, "unlock-gp",         UNLOCK_GP   );
-    getUnlocks(lisp, "unlock-mode",       UNLOCK_MODE );
-    getUnlocks(lisp, "unlock-difficulty", UNLOCK_DIFFICULTY);
-    getUnlocks(lisp, "unlock-kart",       UNLOCK_KART);
+    getUnlocks(root, "unlock-track",      UNLOCK_TRACK);
+    getUnlocks(root, "unlock-gp",         UNLOCK_GP   );
+    getUnlocks(root, "unlock-mode",       UNLOCK_MODE );
+    getUnlocks(root, "unlock-difficulty", UNLOCK_DIFFICULTY);
+    getUnlocks(root, "unlock-kart",       UNLOCK_KART);
 
-    std::vector<std::string> vec;
-    lisp->getVector("depend-on", vec);
-    for(unsigned int i=0; i<vec.size(); i++) addDependency(vec[i]);
-    delete ROOT;
+    if (getFeatures().size() == 0)
+    {
+        error("missing unlocked features");
+    }
+    
+    std::vector< std::string > deps;
+    root->get("depend-on", &deps);
+    for (unsigned int i=0; i<deps.size(); i++)
+    {
+        if (deps[i].size() > 0) addDependency(deps[i]);
+    }
+    delete root;
 
 }   // ChallengeData
 
@@ -134,8 +163,12 @@ void ChallengeData::error(const char *id) const
     std::ostringstream msg;
     msg << "Undefined or incorrect value for '" << id 
         << "' in challenge file '" << m_filename << "'.";
+    
+    std::cerr << "ChallengeData : " << msg.str() << std::endl;
+    
     throw std::runtime_error(msg.str());
 }   // error
+
 // ----------------------------------------------------------------------------
 /** Checks if this challenge is valid, i.e. contains a valid track or a valid
  *  GP. If incorrect data are found, STK is aborted with an error message. 
@@ -156,7 +189,14 @@ void ChallengeData::check() const
     }
     else if(m_major==RaceManager::MAJOR_MODE_GRAND_PRIX)
     {
-        if(!grand_prix_manager->getGrandPrix(m_gp_id))
+        const GrandPrixData* gp = grand_prix_manager->getGrandPrix(m_gp_id);
+        
+        if (gp == NULL)
+        {
+            error("gp");
+        }
+        const bool gp_ok = gp->checkConsistency(false);
+        if (!gp_ok)
         {
             error("gp");
         }
@@ -165,48 +205,56 @@ void ChallengeData::check() const
 
 // ----------------------------------------------------------------------------
 
-
-void ChallengeData::getUnlocks(const lisp::Lisp *lisp, const char* type, 
+void ChallengeData::getUnlocks(const XMLNode *root, const std:: string type,
                                REWARD_TYPE reward)
 {
-    std::vector<std::string> v;
-    v.clear();
-
-    lisp->getVector(type, v);
-    for(unsigned int i=0; i<v.size(); i++)
-    {
-        switch(reward)
-        {
-        case UNLOCK_TRACK:      addUnlockTrackReward     (v[i]        );      break;
-        case UNLOCK_GP:         addUnlockGPReward        (v[i]        );      break;
-        case UNLOCK_MODE:       if(i+1<v.size())
-                                {
-                                    addUnlockModeReward  (v[i], v[i+1]); 
-                                    i++; break;
-                                }
-                                else
-                                    fprintf(stderr, "Unlock mode name missing.\n");
-                                break;
-        case UNLOCK_DIFFICULTY: if(i+1<v.size())
-                                {
-                                    addUnlockDifficultyReward(v[i], v[i+1]); 
-                                    i++;
-                                }
-                                else
-                                    fprintf(stderr, "Difficult name missing.\n");
-                                break;
-        case UNLOCK_KART:       if(i+1<v.size())
-                                {
-                                    addUnlockKartReward(v[i], v[i+1]); 
-                                    i++;
-                                }
-                                else
-                                    fprintf(stderr, "Kart name missing.\n");
-                                break;
-        }   // switch
-    }
-}   // getUnlocks
+    std:: string attrib;
+    root->get(type, &attrib);
     
+    if (attrib . empty()) return;
+
+    //std:: vector< std:: string > data;
+    //std:: size_t space = attrib.find_first_of(' ');
+    //data.push_back( attrib.substr(0, space) );
+    //if( space != std:: string:: npos )
+    //{
+    //    data.push_back( attrib.substr(space, std:: string:: npos) );
+    //}
+
+    switch(reward)
+    {
+    case UNLOCK_TRACK:      addUnlockTrackReward     (attrib        );
+                            break;
+            
+    case UNLOCK_GP:         addUnlockGPReward        (attrib        );
+                            break;
+            
+    case UNLOCK_MODE:       {
+                            const RaceManager::MinorRaceModeType mode =
+                                RaceManager::getModeIDFromInternalName(attrib.c_str());
+                            addUnlockModeReward      (attrib, RaceManager::getNameOf(mode));
+                            break;
+                            }
+    case UNLOCK_DIFFICULTY:
+                            {
+                            irr::core::stringw user_name = "?"; //TODO: difficulty names when unlocking
+                            addUnlockDifficultyReward(attrib, user_name);
+                            break;
+                            }
+    case UNLOCK_KART:       {
+                            const KartProperties* prop = kart_properties_manager->getKart(attrib);
+                            if (prop == NULL)
+                            {
+                                std::cerr << "Challenge refers to kart " << attrib <<
+                                             ", which is unknown. Ignoring reward.\n";
+                                break;
+                            }
+                            irr::core::stringw user_name = prop->getName();
+                            addUnlockKartReward(attrib, user_name);
+                            break;
+                            }
+    }   // switch
+}   // getUnlocks
 // ----------------------------------------------------------------------------
 void ChallengeData::setRace() const
 {
@@ -231,7 +279,6 @@ void ChallengeData::setRace() const
         race_manager->setNumKarts(m_num_karts);
         race_manager->setNumPlayers(1);
         race_manager->setNumLocalPlayers(1);
-        //race_manager->setGrandPrix();
     }
 }   // setRace
 
@@ -243,11 +290,12 @@ bool ChallengeData::raceFinished()
 
     // Single races
     // ------------
-    std::string track_name = RaceManager::getTrack()->getIdent();
-    if(track_name!=m_track_name                    ) return false;    // wrong track
-    if((int)race_manager->getNumKarts()<m_num_karts) return false;    // not enough AI karts
+    World *world = World::getWorld();
+    std::string track_name = world->getTrack()->getIdent();
+    if(track_name!=m_track_name                      ) return false;    // wrong track
+    if((int)world->getNumKarts()<m_num_karts         ) return false;    // not enough AI karts
 
-    Kart* kart = RaceManager::getPlayerKart(0);
+    Kart* kart = world->getPlayerKart(0);
     if(m_energy>0   && kart->getEnergy()  <m_energy  ) return false;  // not enough energy
     if(m_position>0 && kart->getPosition()>m_position) return false;  // too far behind
 
@@ -262,7 +310,7 @@ bool ChallengeData::raceFinished()
     // ---------------------
     // FIXME - encapsulate this better, each race mode needs to be able to specify
     // its own challenges and deal with them
-    LinearWorld* lworld = dynamic_cast<LinearWorld*>(RaceManager::getWorld());
+    LinearWorld* lworld = dynamic_cast<LinearWorld*>(world);
     if(lworld != NULL)
     {
         if(lworld->getLapForKart( kart->getWorldKartId() ) != m_num_laps) return false;         // wrong number of laps
@@ -274,19 +322,23 @@ bool ChallengeData::raceFinished()
 // ----------------------------------------------------------------------------
 bool ChallengeData::grandPrixFinished()
 {
-   // printf("----- checking if GP challenge is solved\n");
+    // Note that we have to call race_manager->getNumKarts, since there
+    // is no world objects to query at this stage.
     if (race_manager->getMajorMode()  != RaceManager::MAJOR_MODE_GRAND_PRIX  ||
         race_manager->getMinorMode()  != m_minor                             ||
         race_manager->getGrandPrix()->getId() != m_gp_id                     ||
         race_manager->getDifficulty()!= m_difficulty                         ||
-        race_manager->getNumKarts()   < (unsigned int)m_num_karts            ||
+        race_manager->getNumberOfKarts() < (unsigned int)m_num_karts         ||
         race_manager->getNumPlayers() > 1) return false;
 
     // check if the player came first.
-    Kart* kart = RaceManager::getPlayerKart(0);
-    const int rank = race_manager->getKartFinalGPRank(kart->getWorldKartId());
+    //assert(World::getWorld() != NULL);
+    // Kart* kart = World::getWorld()->getPlayerKart(0);
+    //const int rank = race_manager->getKartGPRank(kart->getWorldKartId());
+    const int rank = race_manager->getLocalPlayerGPRank(0);
+    
     //printf("getting rank for %s : %i \n", kart->getName().c_str(), rank );
-    if( rank != 0 ) return false;
+    if (rank != 0) return false;
 
     return true;
 }   // grandPrixFinished
