@@ -16,21 +16,32 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#ifndef HEADER_ATTACHMENT_MANAGER_H
-#define HEADER_ATTACHMENT_MANAGER_H
+#ifndef HEADER_ATTACHMENT_MANAGER_HPP
+#define HEADER_ATTACHMENT_MANAGER_HPP
 
-#include <plib/ssg.h>
+#include "irrlicht.h"
 #include "items/attachment.hpp"
+#include "utils/no_copy.hpp"
 
-class AttachmentManager
+/**
+  * \ingroup items
+  */
+class AttachmentManager: public NoCopy
 {
 private:
-    ssgEntity *m_attachments[ATTACH_MAX];
+    scene::IAnimatedMesh *m_attachments[ATTACH_MAX];
+    Material             *m_all_icons [ATTACH_MAX];
 public:
     AttachmentManager() {};
-    ssgEntity *getModel         (attachmentType type) {return m_attachments[type];}
+    /** Returns the mest for a certain attachment.
+     *  \param type Type of the attachment needed. */
+    scene::IAnimatedMesh *getMesh(attachmentType type) const {return m_attachments[type]; }
     void       removeTextures   ();
     void       loadModels       ();
+    /** Returns the icon to display in the race gui if a kart 
+     *  has an attachment. */
+    const Material* 
+               getIcon          (int type) const {return m_all_icons [type];}
 };
 
 extern AttachmentManager *attachment_manager;
