@@ -17,11 +17,19 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#ifndef HEADER_BOWLING_H
-#define HEADER_BOWLING_H
+#ifndef HEADER_BOWLING_HPP
+#define HEADER_BOWLING_HPP
 
-#include "flyable.hpp"
+#include "irrlicht.h"
+using namespace irr;
 
+#include "items/flyable.hpp"
+
+class XMLNode;
+
+/**
+  * \ingroup items
+  */
 class Bowling : public Flyable
 {
 private:
@@ -31,11 +39,11 @@ private:
     
 public:
     Bowling(Kart* kart);
-    static  void init(const lisp::Lisp* lisp, ssgEntity* bowling);
+    static  void init(const XMLNode &node, scene::IMesh *bowling);
     virtual void update(float dt);
     
-	int getExplosionSound() const { return SFXManager::SOUND_BOWLING_STRIKE; }
-	
+    const char* getExplosionSound() const { return "bowling_strike"; }
+    
 };   // Bowling
 
 #endif
