@@ -17,16 +17,38 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#include "rubber_band.hpp"
+#include "items/rubber_band.hpp"
 
-#include "material_manager.hpp"
-#include "race_manager.hpp"
-#include "graphics/scene.hpp"
+#include "graphics/irr_driver.hpp"
+#include "graphics/material_manager.hpp"
 #include "items/plunger.hpp"
 #include "items/projectile_manager.hpp"
 #include "karts/kart.hpp"
 #include "modes/world.hpp"
 #include "physics/physics.hpp"
+#include "race/race_manager.hpp"
+#include "utils/string_utils.hpp"
+
+
+const wchar_t* getPlungerString()
+{
+    const int PLUNGER_STRINGS_AMOUNT = 3;
+
+    RandomGenerator r;
+    const int id = r.get(PLUNGER_STRINGS_AMOUNT);
+
+    switch (id)
+    {
+        //I18N: shown when hit by plunger. %0 is the victim, %1 is the attacker
+        case 0: return _("%0 bites %1's bait");
+        //I18N: shown when hit by plunger. %0 is the victim, %1 is the attacker
+        case 1: return _("%1 latches onto %0 for a free ride");
+        //I18N: shown when hit by plunger. %0 is the victim, %1 is the attacker
+        case 2: return _("%1 tests a tractor beam on %0");
+        default: assert(false); return L"";  // avoid warning about no return value
+    }
+}
+
 
 /** RubberBand constructor. It creates a simple quad and attaches it to the
  *  root(!) of the graph. It's easier this way to get the right coordinates
@@ -36,48 +58,41 @@
  *                 can trigger an explosion)
  *  \param kart    Reference to the kart.
  */
-RubberBand::RubberBand(Plunger *plunger, const Kart &kart)
-          : ssgVtxTable(GL_QUADS, new ssgVertexArray,
-                        new ssgNormalArray,
-                        new ssgTexCoordArray,
-                        new ssgColourArray ), 
+RubberBand::RubberBand(Plunger *plunger, Kart *kart) :
             m_plunger(plunger), m_owner(kart)
 {
+    video::SColor color(77, 179, 0, 0);
+    video::SMaterial m;
+    m.AmbientColor    = color;
+    m.DiffuseColor    = color;
+    m.EmissiveColor   = color;
+    m.BackfaceCulling = false;
+    m_mesh           = irr_driver->createQuadMesh(&m, /*create_one_quad*/ true);
+    m_buffer         = m_mesh->getMeshBuffer(0);
+    m_attached_state = RB_TO_PLUNGER;
+    assert(m_buffer->getVertexType()==video::EVT_STANDARD);
+
+    updatePosition();
+    m_node = irr_driver->addMesh(m_mesh);
 #ifdef DEBUG
-    setName("rubber_band");
+    std::string debug_name = m_owner->getIdent()+" (rubber-band)";
+    m_node->setName(debug_name.c_str());
 #endif
 
-    // The call to update defines the actual coordinates, only the entries are added for now.
-    vertices->add(0, 0, 0); vertices->add(0, 0, 0);
-    vertices->add(0, 0, 0); vertices->add(0, 0, 0);
-    m_attached_state = RB_TO_PLUNGER;
-    updatePosition();
-
-    sgVec3 norm;
-    sgSetVec3(norm, 1/sqrt(2.0f), 0, 1/sqrt(2.0f));
-    normals->add(norm);normals->add(norm);
-    normals->add(norm);normals->add(norm);
-
-    sgVec4 colour;
-    sgSetVec4(colour, 0.7f, 0.0f, 0.0f, 0.3f);
-    colours->add(colour);colours->add(colour);
-    colours->add(colour);colours->add(colour);
-    m_state = new ssgSimpleState();
-    m_state->disable(GL_CULL_FACE);
-    setState(m_state);
-    //setState(material_manager->getMaterial("chrome.rgb")->getState());
-
-    scene->add(this);
 }   // RubberBand
 
 // ----------------------------------------------------------------------------
-/** Removes the rubber band from the scene. Is called when the plunger 
- *  explodes.
- */
-void RubberBand::removeFromScene()
+RubberBand::~RubberBand()
 {
-    scene->remove(this);
-}   // removeFromScene
+    irr_driver->removeNode(m_node);
+
+    // This mesh is self made and so not stored in the
+    // mesh cache. So no reason to call
+    // irr_driver->removeMesh(m_mesh);
+    // But it was implicitly 'grabbed' during creation, so we have
+    // to drop it here.
+    m_mesh->drop();
+}   // RubberBand
 
 // ----------------------------------------------------------------------------
 /** Updates the position of the rubber band. It especially sets the
@@ -86,7 +101,7 @@ void RubberBand::removeFromScene()
  */
 void RubberBand::updatePosition()
 {
-    const Vec3 &k = m_owner.getXYZ();
+    const Vec3 &k = m_owner->getXYZ();
 
     // Get the position to which the band is attached
     // ----------------------------------------------
@@ -104,15 +119,13 @@ void RubberBand::updatePosition()
     // thinner). And call explosion if the band is too long.
     const float hh=.1f;  // half height of the band
     const Vec3 &p=m_end_position;  // for shorter typing
-    float *f = vertices->get(0);
-    f[0] = p.getX()-hh; f[1] = p.getY(); f[2] = p.getZ()-hh;
-    f = vertices->get(1);
-    f[0] = p.getX()+hh; f[1] = p.getY(); f[2] = p.getZ()+hh;
-    f = vertices->get(2);
-    f[0] = k.getX()+hh; f[1] = k.getY(); f[2] = k.getZ()+hh;
-    f = vertices->get(3);
-    f[0] = k.getX()-hh; f[1] = k.getY(); f[2] = k.getZ()-hh;
-    dirtyBSphere();
+    irr::video::S3DVertex* v=(video::S3DVertex*)m_buffer->getVertices();
+    v[0].Pos.X = p.getX()-hh; v[0].Pos.Y=p.getY(); v[0].Pos.Z = p.getZ()-hh;
+    v[1].Pos.X = p.getX()+hh; v[1].Pos.Y=p.getY(); v[1].Pos.Z = p.getZ()+hh;
+    v[2].Pos.X = k.getX()+hh; v[2].Pos.Y=k.getY(); v[2].Pos.Z = k.getZ()+hh;
+    v[3].Pos.X = k.getX()-hh; v[3].Pos.Y=k.getY(); v[3].Pos.Z = k.getZ()-hh;
+    m_buffer->recalculateBoundingBox();
+    m_mesh->setBoundingBox(m_buffer->getBoundingBox());
 }   // updatePosition
 
 // ----------------------------------------------------------------------------
@@ -124,7 +137,7 @@ void RubberBand::updatePosition()
  */
 void RubberBand::update(float dt)
 {
-    if(m_owner.isEliminated())
+    if(m_owner->isEliminated())
     {
         // Rubber band snaps
         m_plunger->hit(NULL);
@@ -134,12 +147,12 @@ void RubberBand::update(float dt)
     }
 
     updatePosition();
-    const Vec3 &k = m_owner.getXYZ();
-    
+    const Vec3 &k = m_owner->getXYZ();
+
     // Check for rubber band snapping
     // ------------------------------
     float l = (m_end_position-k).length2();
-    float max_len = m_owner.getKartProperties()->getRubberBandMaxLength();
+    float max_len = m_owner->getKartProperties()->getRubberBandMaxLength();
     if(l>max_len*max_len)
     {
         // Rubber band snaps
@@ -152,7 +165,7 @@ void RubberBand::update(float dt)
     // ----------------------------
     if(m_attached_state!=RB_TO_PLUNGER)
     {
-        float force = m_owner.getKartProperties()->getRubberBandForce();
+        float force = m_owner->getKartProperties()->getRubberBandForce();
         Vec3 diff   = m_end_position-k;
         
         // detach rubber band if kart gets very close to hit point
@@ -166,7 +179,11 @@ void RubberBand::update(float dt)
         }
         
         diff.normalize();   // diff can't be zero here
-        m_owner.getBody()->applyCentralForce(diff*force);
+        m_owner->getBody()->applyCentralForce(diff*force);
+        m_owner->increaseMaxSpeed(MaxSpeed::MS_INCREASE_RUBBER,
+            m_owner->getKartProperties()->getRubberBandSpeedIncrease(),
+            /*duration*/0.1f, 
+            m_owner->getKartProperties()->getRubberBandFadeOutTime());
         if(m_attached_state==RB_TO_KART)
             m_hit_kart->getBody()->applyCentralForce(diff*(-force));
     }
@@ -185,19 +202,19 @@ void RubberBand::checkForHit(const Vec3 &k, const Vec3 &p)
     short int old_kart_group=0;
 
     // If the owner is being rescued, the broadphase handle does not exist!
-    if(m_owner.getBody()->getBroadphaseHandle())
-        old_kart_group = m_owner.getBody()->getBroadphaseHandle()->m_collisionFilterGroup;
+    if(m_owner->getBody()->getBroadphaseHandle())
+        old_kart_group = m_owner->getBody()->getBroadphaseHandle()->m_collisionFilterGroup;
     m_plunger->getBody()->getBroadphaseHandle()->m_collisionFilterGroup = 0;
-    if(m_owner.getBody()->getBroadphaseHandle())
-        m_owner.getBody()->getBroadphaseHandle()->m_collisionFilterGroup = 0;
+    if(m_owner->getBody()->getBroadphaseHandle())
+        m_owner->getBody()->getBroadphaseHandle()->m_collisionFilterGroup = 0;
 
     // Do the raycast
-    RaceManager::getWorld()->getPhysics()->getPhysicsWorld()->rayTest(k, p, 
-                                                                      ray_callback);
+    World::getWorld()->getPhysics()->getPhysicsWorld()->rayTest(k, p, 
+                                                                ray_callback);
     // Reset collision groups
     m_plunger->getBody()->getBroadphaseHandle()->m_collisionFilterGroup = old_plunger_group;
-    if(m_owner.getBody()->getBroadphaseHandle())
-        m_owner.getBody()->getBroadphaseHandle()->m_collisionFilterGroup = old_kart_group;
+    if(m_owner->getBody()->getBroadphaseHandle())
+        m_owner->getBody()->getBroadphaseHandle()->m_collisionFilterGroup = old_kart_group;
     if(ray_callback.HasHit())
     {
         Vec3 pos(ray_callback.m_hitPointWorld);
@@ -228,6 +245,14 @@ void RubberBand::hit(Kart *kart_hit, const Vec3 *track_xyz)
     {
         m_hit_kart       = kart_hit;
         m_attached_state = RB_TO_KART;
+
+        RaceGUIBase* gui = World::getWorld()->getRaceGUI();
+        irr::core::stringw hit_message;
+        hit_message += StringUtils::insertValues(getPlungerString(),
+                                                 kart_hit->getName().c_str(),
+                                                 m_owner->getName().c_str()
+                                                ).c_str();
+        gui->addMessage(hit_message, NULL, 3.0f, 40, video::SColor(255, 255, 255, 255), false);
         return;
     }
 
