@@ -21,25 +21,25 @@
 #include "modes/linear_world.hpp"
 
 /**
-  * Represents a standard race, i.e. with a start, end and laps.
+  * \brief Represents a standard race, i.e. with a start, end and laps.
   * Used in Grand Prix, Quick Race and Time Trial.
+  * \ingroup modes
   */
 class StandardRace : public LinearWorld
 {
-public:
-    StandardRace();
-    virtual ~StandardRace();
-    
+protected:
     // clock events
-    virtual void onGo();
-    virtual void terminateRace();
+    virtual bool  isRaceOver();
+
+public:
+                  StandardRace();
+    virtual      ~StandardRace() {};
     
     // overriding World methods
-    virtual void update(float delta);
-    virtual void restartRace();
-    virtual void getDefaultCollectibles(int& collectible_type, int& amount);
-    virtual bool enableBonusBoxes();
-    virtual std::string getInternalCode() const;
+    virtual void  getDefaultCollectibles(int& collectible_type, int& amount);
+    virtual bool  haveBonusBoxes();
+    virtual std::string 
+                  getIdent() const;
 };
 
 #endif
