@@ -17,8 +17,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#ifndef HEADER_CONNECT_MESSAGE_H
-#define HEADER_CONNECT_MESSAGE_H
+#ifndef HEADER_CONNECT_MESSAGE_HPP
+#define HEADER_CONNECT_MESSAGE_HPP
 
 #include <string>
 
