@@ -17,8 +17,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#ifndef HEADER_NETWORK_KART_H
-#define HEADER_NETWORK_KART_H
+#ifndef HEADER_NETWORK_KART_HPP
+#define HEADER_NETWORK_KART_HPP
 
 #include "karts/kart.hpp"
 
@@ -31,5 +31,6 @@ public:
                      const btTransform& init_transform,
                      int global_player_id);
     void setControl(const KartControl& kc);
+    virtual bool isNetworkKart() const { return true; }
 };   // NetworkKart
 #endif
