@@ -17,8 +17,8 @@
 //  along with this program; if not, write to the Free Software
 //  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-#ifndef HEADER_WORLD_LOADED_H
-#define HEADER_WORLD_LOADED_H
+#ifndef HEADER_WORLD_LOADED_HPP
+#define HEADER_WORLD_LOADED_HPP
 
 #include "network/message.hpp"
 
